"""Definition of the FilesystemFolder content type and associated schemata and
other logic.

This file contains a number of comments explaining the various lines of
code. Other files in this sub-package contain analogous code, but will 
not be commented as heavily.

Please see README.txt for more information on how the content types in
this package are used.
"""

import re
from zope.interface import implements
from zope.component import adapter, getMultiAdapter, getUtility

from zope.app.container.interfaces import INameChooser

from Acquisition import aq_inner, aq_parent

from Products.Archetypes import atapi
from Products.Archetypes.interfaces import IObjectInitializedEvent

from Products.ATContentTypes.content import folder
from Products.ATContentTypes.content.schemata import finalizeATCTSchema

from fui.fsbrowse.interfaces import IFilesystemFolder
from fui.fsbrowse.config import PROJECTNAME
from fui.fsbrowse import FsBrowseMessageFactory as _



# This is the Archetypes schema, defining fields and widgets. We extend
# the one from ATContentType's ATFolder with our additional fields.
FilesystemFolderSchema = folder.ATFolderSchema.copy() + atapi.Schema((
	atapi.StringField("path",
		required = True,
		searchable = False,
		storage = atapi.AnnotationStorage(),
		widget = atapi.StringWidget(
			label = u"Path",
			description = u"Path to a folder in the filesystem."),
		),
	))

# We want to ensure that the properties we use as field properties (see
# below), use AnnotationStorage. Without this, our property will conflict
# with the attribute with the same name that is being managed by the default
# attributestorage
FilesystemFolderSchema['title'].storage = atapi.AnnotationStorage()

# Calling this re-orders a few fields to comply with Plone conventions.
finalizeATCTSchema(FilesystemFolderSchema, folderish=True, moveDiscussion=False)



class FilesystemFolder(folder.ATFolder):
	""" Contains multiple fsbrowse reservations. """
	implements(IFilesystemFolder)
	
	# The portal type name must be set here, matching the one in types.xml
	# in the GenericSetup profile
	portal_type = "FilesystemFolder"
	
	# This enables Archetypes' standard title-to-id renaming machinery
	# If you need different semantics, it's possible to override the method
	# _renameAfterCreation() from BaseObject
	_at_rename_after_creation = True
	
	# We then associate the schema with our content type
	schema = FilesystemFolderSchema


# This line tells Archetypes about the content type
atapi.registerType(FilesystemFolder, PROJECTNAME)



from os import listdir, sep
from os.path import isdir, join, exists, dirname
import codecs
import re

from Acquisition import aq_inner
from Products.Five.browser import BrowserView
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from Products.CMFCore.utils import getToolByName
from plone.memoize.instance import memoize
from Products.statusmessages.interfaces import IStatusMessage



EMBEDDABLE_PATT = re.compile("(\.txt|\.rst|\.md|README)$", re.IGNORECASE)
IMAGES_PATT = re.compile("(\.png|\.jpg|\.jpeg|\.jpe|\.gif)$", re.IGNORECASE)
SHORT_FILENAME_LEN = 18


class FsItem(object):
	def __init__(self, requestedDir, basename, shortNames=False, isDir=False,
			displayname=None):
		self.requestedDir = requestedDir
		self.basename = basename
		self._isDir = isDir
		self.isSelected = False
		displayname = displayname or basename
		if shortNames and len(displayname) > SHORT_FILENAME_LEN:
			displayname = displayname[0:SHORT_FILENAME_LEN-3] + "..."
		self.displayname = displayname

		if self.isImage():
			self.displayname = self.displayname.rsplit(".", 1)[0]

		if requestedDir != "":
			path = "%s/%s" % (requestedDir, basename)
		else:
			path = basename
		self.path = path
		self.viewurl = "view?path=" + path
		self.rawurl = "raw?path=" + path
		self.downloadurl = "download?path=" + path

	def getDownloadUrl(self):
		return self.downloadurl

	def getRawUrl(self):
		return self.rawurl

	def isEmbeddable(self):
		return not self.isDir() and EMBEDDABLE_PATT.search(self.basename)

	def isImage(self):
		return not self.isDir() and IMAGES_PATT.search(self.basename)

	def isDir(self):
		return self._isDir

	def getBasename(self):
		return self.basename

	def getDisplayname(self):
		return self.displayname

	def getClasses(self):
		if self.isEmbeddable():
			classes = "typeEmbeddable"
		elif self.isImage():
			classes = "typeImage"
		else:
			classes = "typeBin"

		if self.isSelected:
			classes += " selectedFile"
		return classes

	def __cmp__(self, other):
		return cmp(self.displayname, other.displayname)



class FsFileBase(BrowserView, FsItem):

	def __init__(self, *args, **kwargs):
		""" Get the virtual path of the requested file or folder from
		the GET form.

		Sets the following variables:

			self.basepath:
				The path configured by the manager for this
				FilesystemFolder.
			self.requestedPath:
				The path requested by the user.
			self.requestedDir:
				The directory part of the path requested by the user.
			self.requestedDir:
			self.basename:
				The last component of self.requestedPath.
			self.dirpath:
				The absolute path to the requested directory on disk.
			self.filepath:
				The absolute path to the requested file on disk. Is None if the
				requested object is a directory.
		"""
		BrowserView.__init__(self, *args, **kwargs)
		self.ctx = aq_inner(self.context)

		self.basepath = self.ctx.getPath()
		requestedPath = self.request.form.get("path", "")
		if requestedPath:
			basename = requestedPath.split("/")[-1]
			path = join(self.basepath, requestedPath.replace("/", sep))
			if not exists(path):
				IStatusMessage(self.request).addStatusMessage(
					"Invalid path: %s. Redirected to rootdirectory." % requestedPath,
					type='error')
				path = self.basepath
				requestedPath = ""
		else:
			path = self.basepath
			basename = self.ctx.Title()
		self.requestedPath = requestedPath


		if path.endswith("/"):
			path = path[:-1]
		if isdir(path):
			self.filepath = None
			self.dirpath = path
		else:
			self.filepath = path
			self.dirpath = dirname(path)

		requestedDir = "/".join(requestedPath.split("/")[:-1])
		FsItem.__init__(self, requestedDir, basename, isDir=isdir(path))



from zope.interface import Interface
from zope import schema

from zope.app.container.constraints import contains

from fui.fsbrowse import FsBrowseMessageFactory as _


class IFilesystemFolder(Interface):
	""" A filesystem folder. """



"""Common configuration constants
"""

PROJECTNAME = "fui.fsbrowse"
PROMOTIONS_PORTLET_COLUMN = u"plone.rightcolumn"

# This maps portal types to their corresponding add permissions.
# These are referenced in the root product __init__.py, during
# Archetypes/CMF type initialisation. The permissions here are
# also defined in content/configure.zcml, so that they can be
# looked up as a Zope 3-style IPermission utility.

# We prefix the permission names with our product name to group
# them sensibly. This is good practice, because it makes it
# easier to find permissions in the Security tab in the ZMI.

ADD_PERMISSIONS = {
	"FilesystemFolder" : "fui.fsbrowse: Add FilesystemFolder",
}



from os import listdir, sep
from os.path import isdir, join, exists, dirname
import codecs

from Acquisition import aq_inner
from Products.Five.browser import BrowserView
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from Products.CMFCore.utils import getToolByName
from plone.memoize.instance import memoize
from Products.statusmessages.interfaces import IStatusMessage

from fsfile_base import FsFileBase, FsItem



class FsBrowse(FsFileBase, FsItem):
	__call__ = ViewPageTemplateFile('fsbrowse.pt')


	def __init__(self, *args, **kwargs):
		FsFileBase.__init__(self, *args, **kwargs)
		self._getFiletrail()
		self._parseFsFolder()


	def _getFiletrail(self):
		self.fileTrail = []
		if self.requestedPath:
			self.fileTrail.append(FsItem("", "", isDir=True, displayname=self.ctx.Title()))
			s = ""
			for x in self.requestedPath.split(sep)[:-1]:
				i = FsItem(s, x)
				self.fileTrail.append(i)
				s = i.path
		#print [x.path for x in self.fileTrail]


	def _parseFsFolder(self):
		if self.isDir():
			requestedDir = self.requestedPath
			shortNames = False
		else:
			requestedDir = self.requestedDir
			shortNames = True

		self.files = []
		self.folders = []
		for basename in listdir(self.dirpath):
			item = FsItem(requestedDir, basename, shortNames)
			if basename == self.basename:
				item.isSelected = True
			if isdir(join(self.dirpath, basename)):
				self.folders.append(item)
			else:
				self.files.append(item)

		self.folders.sort()
		self.files.sort()


	def getFolders(self):
		return self.folders

	def getFiles(self):
		return self.files

	def getUtf8Data(self):
		return codecs.open(self.filepath, mode="rb",
				encoding="utf-8", errors="replace").read()

	def getFileTrail(self):
		return self.fileTrail



from setuptools import setup, find_packages
import os

version = '0.3'

setup(name='fui.fsbrowse',
	version=version,
	description="",
	long_description=open("README.rst").read(),
	# Get more strings from http://www.python.org/pypi?%3Aaction=list_classifiers
	classifiers=[
		"Framework :: Plone",
		"Programming Language :: Python",
		"Topic :: Software Development :: Libraries :: Python Modules",
	],
	keywords='',
	author='Espen A. Kristiansen',
	author_email='post@espenak.net',
	url='http://github.com/espenak/fui.fsbrowse',
	license='GPL',
	packages=find_packages(exclude=['ez_setup']),
	namespace_packages=['fui'],
	include_package_data=True,
	zip_safe=False,
	install_requires=[
		'setuptools',
		# -*- Extra requirements: -*-
	],
	entry_points="""
	# -*- Entry points: -*-

	[distutils.setup_keywords]
	paster_plugins = setuptools.dist:assert_string_list

	[egg_info.writers]
	paster_plugins.txt = setuptools.command.egg_info:write_arg
	""",
	paster_plugins = ["ZopeSkel"],
)



from zope.publisher.browser import BrowserView

from fsfile_base import FsFileBase
from mimetypes import guess_type, add_type


add_type("text/plain", ".rst")
add_type("text/plain", ".md")
add_type("text/plain", ".mkd")
add_type("text/plain", ".textile")


class RawFileView(FsFileBase):
	def __call__(self):
		self.setHeaders()
		return open(self.filepath, "rb").read()

	def setHeaders(self):
		if self.basename == "README":
			mtype = "text/plain"
			encoding = None
		else:
			mtype, encoding = guess_type(self.basename)

		if mtype == "text/plain":
			mtype = mtype + "; charset=utf-8"

		self.request.response.setHeader(
				"Content-Type", mtype)
		if encoding:
			self.request.response.setHeader(
					"Content-Encoding", encoding)


class RawFileDownload(RawFileView):
	def __call__(self):
		self.setHeaders()
		self.request.response.setHeader(
				'Content-Disposition',
				'attachment; filename=%s' % self.basename)
		return open(self.filepath, "rb").read()
